from pydantic import Field
from koil import koilable
from koil.composition import Composition
from mikro.datalayer import DataLayer
from mikro.rath import MikroRath


@koilable(add_connectors=True)
class Mikro(Composition):
    """ The Mikro Composition
    
    This composition provides a datalayer and a rath for interacting with the
    mikro api and beyond

    You shouldn't need to create this directly, instead use the builder functions
    to generate a new instance of this composition.

    ```python

    from mikro import Mikro

    async def aget_token():
        return "XXXX"

    m = Mikro(
        datalayer= DataLayer(endpoint_url="s3.amazonaws.com", access_key="XXXX", secret_key="XXXX"),
        mikro = MikroRath(link=MikroLinkComposition(auth=AuthTokenLink(token_loader=aget_token)))),
    )
    ```
    """


    datalayer: DataLayer
    rath: MikroRath = Field(default_factory=MikroRath)

    def _repr_html_inline_(self):
        return f"<table><tr><td>datalayer</td><td>{self.datalayer._repr_html_inline_()}</td></tr><tr><td>rath</td><td>{self.rath._repr_html_inline_()}</td></tr></table>"



""" Strucutre Registration

This module is autoimported by arkitekt. It registers the default structure types with the arkitekt
structure-registry so that they can be used in the arkitekt app without having to import them.

You can of course overwrite this in your app if you need to expand to a more complex query.

"""


try:
    from rekuest.structures.default import get_default_structure_registry
    from rekuest.widgets import SearchWidget, ImageReturnWidget
    from rekuest.widgets import CustomReturnWidget
    from mikro.api.schema import *

    structure_reg = get_default_structure_registry()
    structure_reg.register_as_structure(
        RepresentationFragment,
        identifier="@mikro/representation",
        expand=aexpand_representation,
        default_widget=SearchWidget(
            query=Search_representationQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        MetricFragment,
        identifier="@mikro/metric",
        expand=aexpand_metric,
        default_widget=None,
        default_returnwidget=CustomReturnWidget(hook="metric", ward="mikro"),
    )
    structure_reg.register_as_structure(
        SampleFragment,
        identifier="@mikro/sample",
        expand=aexpand_sample,
        default_widget=SearchWidget(
            query=Search_sampleQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        TableFragment,
        identifier="@mikro/table",
        expand=aexpand_table,
        default_widget=SearchWidget(
            query=Search_tablesQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        ExperimentFragment,
        identifier="@mikro/experiment",
        expand=aexpand_experiment,
        default_widget=SearchWidget(
            query=Search_experimentQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        ThumbnailFragment,
        identifier="@mikro/thumbnail",
        expand=aexpand_thumbnail,
        default_widget=SearchWidget(
            query=Search_thumbnailsQuery.Meta.document, ward="mikro"
        ),
        default_returnwidget=ImageReturnWidget(
            query=Image_for_thumbnailQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        OmeroFileFragment,
        identifier="@mikro/omerofile",
        expand=aexpand_omerofile,
        default_widget=SearchWidget(
            query=Search_omerofileQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        ROIFragment,
        identifier="@mikro/roi",
        expand=aexpand_roi,
        default_widget=SearchWidget(query=Search_roisQuery.Meta.document, ward="mikro"),
    )
    structure_reg.register_as_structure(
        FeatureFragment,
        identifier="@mikro/feature",
        expand=aexpand_feature,
        default_widget=SearchWidget(
            query=Search_featuresQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        LabelFragment,
        identifier="@mikro/label",
        expand=aexpand_label,
        default_widget=SearchWidget(
            query=Search_labelsQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        PositionFragment,
        identifier="@mikro/position",
        expand=aexpand_position,
        default_widget=SearchWidget(
            query=Search_positionsQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        StageFragment,
        identifier="@mikro/stage",
        expand=aexpand_stage,
        default_widget=SearchWidget(
            query=Search_stagesQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        DatasetFragment,
        identifier="@mikro/dataset",
        expand=aexpand_dataset,
        default_widget=SearchWidget(
            query=Search_datasetsQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        ObjectiveFragment,
        identifier="@mikro/objective",
        expand=aexpand_objective,
        default_widget=SearchWidget(
            query=Search_objectivesQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        ModelFragment,
        identifier="@mikro/model",
        expand=aexpand_model,
        default_widget=SearchWidget(
            query=Search_modelsQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        ContextFragment,
        identifier="@mikro/context",
        expand=aexpand_context,
        default_widget=SearchWidget(
            query=Search_contextsQuery.Meta.document, ward="mikro"
        ),
    )
    structure_reg.register_as_structure(
        LinkFragment,
        identifier="@mikro/link",
        expand=aexpand_link,
        default_widget=SearchWidget(
            query=Search_linksQuery.Meta.document, ward="mikro"
        ),
    )


except ImportError:
    structure_reg = None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mikro',
 'mikro.api',
 'mikro.cli',
 'mikro.contrib',
 'mikro.contrib.fakts',
 'mikro.links']

package_data = \
{'': ['*'], 'mikro.cli': ['schemas/*']}

install_requires = \
['dask>=2022.2.1,<2023.0.0',
 'pandas>=1.3.4,<2.0.0',
 'rath>=0.3.3',
 's3fs>=2022.2.0,<2023.0.0',
 'xarray>=2022.3.0,<2023.0.0',
 'zarr>=2.11.1,<3.0.0']

extras_require = \
{'complete:python_version >= "3.9"': ['pyarrow>=6.0.1,<7.0.0',
                                      'turms>=0.2.3,<0.3.0'],
 'table:python_version >= "3.9"': ['pyarrow>=6.0.1,<7.0.0'],
 'turms:python_version >= "3.9"': ['turms>=0.2.3,<0.3.0']}

setup_kwargs = {
    'name': 'mikro',
    'version': '0.3.35',
    'description': 'images for arkitekt',
    'long_description': 'None',
    'author': 'jhnnsrs',
    'author_email': 'jhnnsrs@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Default Widgets

This Module provides default widgets that can be used with the mikro app in an arkitekt
context.

Attributes:
    MY_TOP_REPRESENTATIONS (SearchWidget): The top representations for the currently active user
    MY_TOP_SAMPLES (SearchWidget): The top samples for the currently active user
"""


try:
    from mikro.api.schema import (
        Search_representationQuery,
        Search_sampleQuery,
        Search_tagsQuery,
    )
    from rekuest.widgets import SearchWidget

    MY_TOP_REPRESENTATIONS = SearchWidget(
        query=Search_representationQuery.Meta.document, ward="mikro"
    )

    MY_TOP_SAMPLES = SearchWidget(query=Search_sampleQuery.Meta.document, ward="mikro")

    TAGS_WIDGET = SearchWidget(query=Search_tagsQuery.Meta.document, ward="mikro")

except:
    pass



import argparse
from enum import Enum
import os
from rich import get_console


directory = os.getcwd()

file_path = os.path.dirname(__file__)


class MikroOptions(str, Enum):
    GEN = "gen"


v1_file= """
projects:
  mikro:
    schema: http://localhost:8080/graphql
    documents: graphql/mikro/*/**.graphql
    extensions:
      turms:
        out_dir: mikro/api
        freeze:
          enabled: true
        stylers:
          - type: turms.stylers.default.DefaultStyler
          - type: turms.stylers.appender.AppenderStyler
            append_fragment: "Fragment"
        plugins:
          - type: turms.plugins.enums.EnumsPlugin
          - type: turms.plugins.inputs.InputsPlugin
          - type: turms.plugins.fragments.FragmentsPlugin
          - type: turms.plugins.operations.OperationsPlugin
          - type: turms.plugins.funcs.FuncsPlugin
            global_kwargs:
              - type: mikro.rath.MikroRath
                key: rath
                description: "The mikro rath client"
            definitions:
              - type: subscription
                is_async: True
                use: mikro.funcs.asubscribe
              - type: query
                is_async: True
                use: mikro.funcs.aexecute
              - type: mutation
                is_async: True
                use: mikro.funcs.aexecute
              - type: subscription
                use: mikro.funcs.subscribe
              - type: query
                use: mikro.funcs.execute
              - type: mutation
                use: mikro.funcs.execute
        processors:
          - type: turms.processors.black.BlackProcessor
        scalar_definitions:
          XArrayInput: mikro.scalars.XArrayInput
          File: mikro.scalars.File
          ImageFile: mikro.scalars.File
          Upload: mikro.scalars.Upload
          ModelData: mikro.scalars.ModelData
          ModelFile: mikro.scalars.ModelFile
          ParquetInput: mikro.scalars.ParquetInput
          Store: mikro.scalars.Store
          Parquet: mikro.scalars.Parquet
          ID: rath.scalars.ID
          MetricValue: mikro.scalars.MetricValue
          FeatureValue: mikro.scalars.FeatureValue
        additional_bases:
          Representation:
            - mikro.traits.Representation
          Table:
            - mikro.traits.Table
          Omero:
            - mikro.traits.Omero
          Objective:
            - mikro.traits.Objective
          Position:
            - mikro.traits.Position
          Stage:
            - mikro.traits.Stage
          ROI:
            - mikro.traits.ROI
          InputVector:
            - mikro.traits.Vectorizable
"""


def main(script: MikroOptions, documents: str, codefile: str):
    """The main entrypoint for the CLI"""

    app_directory = os.getcwd()

    if script == MikroOptions.GEN:
        try:

            from turms.run import (
                gen,
                load_projects_from_configpath,
                generate,
                write_code_to_file,
                GraphQLProject,
                GeneratorConfig,
            )
            from turms.config import Extensions, GeneratorConfig, FreezeConfig
            from turms.plugins.enums import EnumsPlugin
            from turms.plugins.inputs import InputsPlugin
            from turms.plugins.fragments import FragmentsPlugin
            from turms.plugins.operations import OperationsPlugin
            from turms.plugins.funcs import FuncsPlugin
        except ImportError:
            raise ImportError(
                "Please install turms to use the gen command. `pip install turms`"
            )

        
        project = GraphQLProject(
          schema= os.path.join(file_path, "schemas/v1.graphql"),
          documents= documents,
          extensions=Extensions(
            turms=GeneratorConfig(
              out_dir=app_directory,
              generated_name=codefile,
              freeze=FreezeConfig(
                enabled=True
              ),
              plugins=[
                {"type": "turms.plugins.enums.EnumsPlugin"},
                {"type": "turms.plugins.inputs.InputsPlugin"},
                {"type": "turms.plugins.fragments.FragmentsPlugin"},
                {"type": "turms.plugins.operations.OperationsPlugin"},
                {"type": "turms.plugins.funcs.FuncsPlugin"},
              ]

  
          )
          )

        )


        generated_code = generate(project)

        get_console().print(f"-------------- Generating project: mikro --------------")

        write_code_to_file(
            generated_code,
            app_directory,
            codefile,
        )

        get_console().print("Sucessfull!! :right-facing_fist::left-facing_fist:")


def entrypoint():
    parser = argparse.ArgumentParser(description="Say hello")
    parser.add_argument("script", type=MikroOptions, help="The Script Type")
    parser.add_argument(
        "documents",
        type=str,
        help="The gloab to documents",
        nargs="?",
        default="*.graphql",
    )
    parser.add_argument(
        "codefile",
        type=str,
        help="The gloab to documents",
        nargs="?",
        default="generated.py",
    )
    args = parser.parse_args()
    try:
        main(
            script=args.script,
            documents=args.documents,
            codefile=args.codefile,
        )
    except Exception as e:
        get_console().print_exception()


if __name__ == "__main__":
    entrypoint()
